'use strict'

_ = require "lodash"
fs = require 'fs-extra'
child_process = require 'child_process'
async = require 'async'
chalk = require 'chalk'

Producer = require 'class_node_producer'

logger = require "omg-logger"

isWatching = false
watcher = false
ko = require 'knockout'

debug = require("debug")("omg-callinfo")

readFileIfExists = (path, cb) ->
  fs.exists path, (itExists) ->
    if itExists
      fs.readFile path, (err, contents) ->
        contents = contents.toString()
        cb err, contents
    else
      cb null, null

readJsonIfExists = (path, cb) ->
  fs.exists path, (itExists) ->
    if itExists
      fs.readJson path, (err, contents) ->
        cb null, contents
    else
      cb null, null

logToJson = (path, cb) ->
  readFileIfExists path, (err, log) ->
    lines = []
    if log
      lines = log.split /[\n\r]+/
      lines = (line.replace(/\'/g, '"') for line in lines when line)
    cb err, lines

getChannelLogs = (chan, cb) ->
  logs = {}
  async.parallel
    inbound: (callback) ->
      logToJson "/tmp/inbound_call_#{chan.channelIndex}.log", callback
    outbound: (callback) ->
      logToJson "/tmp/outbound_call_#{chan.channelIndex}.log", callback
    ISDN: (callback) ->
      readJsonIfExists "/tmp/isdn-call-#{chan.channelIndex}.txt", callback

    channel: (callback) ->
      callback null, chan.channelIndex
    name: (callback) ->
      callback null, chan.name
  , cb

class Callinfo extends Producer
  constructor: ->
    super "callinfo", 15 * 1000

    @start = =>
      debug "starting"
      @debug = debug
      watchDir = "/tmp/"

      ###
      checks if changed file was BChannelState, then runs getChannelLogs.lua and sends result through zmq to prod manager
      ###
      trigger = ko.observable("").extend({rateLimit:2000, notify: "always"})

      onSomethingChanged = ko.computed (filename) =>
        filename = trigger()
        @requestData {producer: "channels", action: "getStatus"}, (err, status, data) =>
          debug "channels responded"
          # convert dchannels to array
          dchannels = (val for key, val of data)
          dchannels = _.sortBy dchannels, "channelIndex"

          async.map dchannels, getChannelLogs, (err, allLogs) =>
            debug chalk.red "error", err if err
            returnObject = {}
            _.each allLogs, (log) ->
              returnObject[log.name] = log
            @sendData returnObject

      watcher = fs.watch watchDir, (event, filename) ->
        if filename.lastIndexOf("BChannelState", 0) is 0 or filename.lastIndexOf("isdn-call-", 0) is 0 or filename.lastIndexOf("inbound_call", 0) is 0 or filename.lastIndexOf("outbound_call", 0) is 0
          trigger(filename)

    #start immediately
    @start()

    @stop = ->
      # watcher.close();
      # isWatching = false;

  subscribe: =>
    if not isWatching
      isWatching = true
      # @start()
    super

module.exports = new Callinfo()

